package com.revolsys.spring.config;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.springframework.beans.BeansException;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.config.RuntimeBeanNameReference;
import org.springframework.beans.factory.config.TypedStringValue;

public class SetBeanProperties implements BeanFactoryPostProcessor,
  InitializingBean {
  private Map<String, String> beanPropertyNames = new LinkedHashMap<String, String>();

  private String targetTypeName;

  private Object value;

  private String ref;

  private Object propertyValue;

  public void addBeanPropertyName(final String beanName,
    final String propertyName) {
    beanPropertyNames.put(beanName, propertyName);
  }

  @Override
  public void afterPropertiesSet() throws Exception {
    assert (value != null & ref != null) : "Cannot have a value and a ref";
    if (ref != null) {
      propertyValue = new RuntimeBeanNameReference(ref);
    } else if (value != null) {
      if (value instanceof String) {
        if (targetTypeName == null) {
          propertyValue = new TypedStringValue((String)value);
        } else {
          propertyValue = new TypedStringValue((String)value, targetTypeName);
        }
      } else {
        propertyValue = value;
      }
    }
  }

  public Map<String, String> getBeanPropertyNames() {
    return beanPropertyNames;
  }

  protected Object getPropertyValue() {
    return propertyValue;
  }

  public String getRef() {
    return ref;
  }

  public String getTargetTypeName() {
    return targetTypeName;
  }

  public Object getValue() {
    return value;
  }

  @Override
  public void postProcessBeanFactory(
    final ConfigurableListableBeanFactory beanFactory) throws BeansException {
    for (final Entry<String, String> beanPropertyName : beanPropertyNames.entrySet()) {
      String beanName = beanPropertyName.getKey();
      final String[] aliases = beanFactory.getAliases(beanName);
      if (aliases.length > 0) {
        beanName = aliases[0];
      }
      final String propertyName = beanPropertyName.getValue();
      final BeanDefinition beanDefinition = beanFactory.getBeanDefinition(beanName);
      beanDefinition.setLazyInit(false);
      final MutablePropertyValues propertyValues = beanDefinition.getPropertyValues();
      propertyValues.add(propertyName, propertyValue);
    }
  }

  public void setBeanPropertyNames(final Map<String, String> beanPropertyNames) {
    this.beanPropertyNames = beanPropertyNames;
  }

  protected void setPropertyValue(final Object propertyValue) {
    this.propertyValue = propertyValue;
  }

  public void setRef(final String ref) {
    this.ref = ref;
  }

  public void setTargetTypeName(final String targetTypeName) {
    this.targetTypeName = targetTypeName;
  }

  public void setValue(final Object value) {
    this.value = value;
  }
}
